///////////////////////////////////////////////////////////////////////////////
// Objects
//
// Single object
// -------------
// Single objects require the following items in the definition string,
//  separated by spaces.
//
//  0: Category (must be 's' or 'single' to define this as a single object)
//  1: Name (name for object; must be unique)
//  2: File (filename for model; may be .dts or .dif file)
//  3: MaxSlope (maximum slope to place on; degrees; default = 40)
//  4: ZOffset (distance to offset from terrain;
//              takes slope into account; default = 0)
//  5: SlopeWithTerrain (fraction of terrain slope off of vertical;
//                       continuous range, examples:
//                       0.0 = ignore terrain/stay vertical;
//                       0.5 = halfway between vertical and
//                             perpendicular to the surface;
//                       1.0 = conform completely to terrain angle;
//                       R   = random value between vertical and
//                             perpendicular to the surface;
//                       values 0.0 to 1.0 or R; default = 0)
//  6: MinScale (minimum for random scale range; default = 0.8)
//  7: MaxScale (maximum for random scale range; default = 1.4)
//
// Compound object
// ---------------
// Compound objects require the following items in the definition string,
//  separated by spaces. Fields 10-19 are repeated for each extra
//  (non-base) child object in the compound object.
//
//   0: Category (must be 'c' or 'compound' to define
//                this as a compound object)
// 1-7: Base fields (identical to the fields for a single;
//                   defines the base object)
//   8: Number of child objects (default = 0)
//   9: ReachForVertical (not implemented yet, but you must
//                        include a value atm)
//  10: File (filename for model; may be .dts or .dif file)
//  11: x positional offset (from base object's computed origin)
//  12: y positional offset (from base object's computed origin)
//  13: z positional offset (from base object's computed origin)
//  14: x rotational offset (from base object's computed rotation;
//                           values -179 to 180 or R = randomize)
//  15: y rotational offset (from base object's computed rotation;
//                           values -179 to 180 or R = randomize)
//  16: z rotational offset (from base object's computed rotation;
//                           values -179 to 180 or R = randomize)
//  17: x scale offset (from base object's scale as computed using 6 & 7)
//  18: y scale offset (from base object's scale as computed using 6 & 7)
//  19: z scale offset (from base object's scale as computed using 6 & 7)
//
// Variety object
// ---------------
// Variety objects require the following items in the definition string,
//  separated by spaces. Fields 3-4 are repeated for each child object
//  in this variety object.
//
//  0: Category (must be 'v' or 'variety' to define this as a variety object)
//  1: Name (name for object; must be unique)
//  2: Number of child objects (default = 0)
//  3: Child object name (must name a single or compound object that is
//                        defined; beware of specifying another variety - you
//                        can do so, but if you create a logical loop, Tribes
//                        will choke since the object type uses recursion to
//                        place objects)
//  4: Child object weight (must be numeric; all weights are translated into
//                          relative percentages)


// Variety recursion (loops) -
//
//   beware of this situation (will loop infinitely):
//
// addRndObj("v FoPShrubs1 2 FoPS23 1 FoPTrees1 2");
// addRndObj("v FoPTrees1 2 FoPT18a 1 FoPShrubs1 1");
//
//   so will this:
//
// addRndObj("v FoPShrubs1 2 FoPS23 1 FoPShrubs1 2");
//
//  very, very bad...



// sample objects
addRndObj("s TestSingle borg17.dts 60 -3.0 1.5 0.8 1.5");
addRndObj("c TestCompound borg1.dts 60 0.35 1 0.5 2" SPC
                         "2 0" SPC
                         "borg1.dts 0 0 0.25 0 0 R 0.75 0.75 1.00" SPC
                         "borg1.dts 0 0 0.50 0 0 R 0.50 0.50 1.25");
addRndObj("v TestVariety 3 Crate1 25 Crate2 25 Crate3 50");


// Boiderm organics (badlands)
addRndObj("s BiodermPlant3 xorg3.dts 40 -0.5");
addRndObj("s BiodermPlant4 xorg4.dts");
addRndObj("s BiodermPlant5 xorg5.dts");
addRndObj("s BiodermPlant20 xorg20.dts");
addRndObj("s BiodermPlant21 xorg21.dts");

// Blood Eagle organics (lush)
addRndObj("s LushPlant1 borg1.dts 40 0.35 1 0.5 2");
addRndObj("s LushPlant2 borg2.dts");
addRndObj("s LushPlant4 borg4.dts");
addRndObj("s LushPlant5 borg5.dts 40 0.0 1 1 1.5");
addRndObj("c LushPlantA borg1.dts 60 0.35 1 0.5 2" SPC
                         "2 0" SPC
                         "borg1.dts 0 0 0.25 0 0 R 0.75 0.75 1.00" SPC
                         "borg1.dts 0 0 0.50 0 0 R 0.50 0.50 1.25");

addRndObj("s LushGrass6 borg6.dts");
addRndObj("s LushGrass7 borg7.dts");
addRndObj("s LushGrass8 borg8.dts");

addRndObj("s LushPlant12 borg12.dts");
addRndObj("s LushPlant13 borg13.dts");

addRndObj("s LushLgTree16 borg16.dts 20 -3.0 0 0.8 1.5");
addRndObj("s LushSmTree17 borg17.dts 20 -3.0 0.5 0.8 1.5");
addRndObj("s LushLgTree18 borg18.dts 20 -3.0 0 0.8 1.5");
addRndObj("s LushLgTree19 borg19.dts 20 -3.0 0 0.8 1.5");

addRndObj("s LushShrub20 borg20.dts");
addRndObj("s LushShrub23 borg23.dts");
addRndObj("s LushShrub25 borg25.dts");
addRndObj("s LushShrub31 borg31.dts");
addRndObj("s LushShrub32 borg32.dts");
addRndObj("s LushShrub33 borg33.dts");
addRndObj("s LushShrub34 borg34.dts");

// Phoenix organics (desert)
addRndObj("s DesertPlant1 porg1.dts");
addRndObj("s DesertPlant2 porg2.dts");
addRndObj("s DesertPlant3 porg3.dts");
addRndObj("s DesertPlant4 porg4.dts");
addRndObj("s DesertPlant5 porg5.dts 25 -0.2 1 0.6 1.0");
addRndObj("s DesertPlant6 porg6.dts");
addRndObj("s DesertPlant20 porg20.dts");
addRndObj("s DesertPlant22 porg22.dts 25 0.1 1 0.8 1.4");

// Starwolf organics (ice)
addRndObj("s IceTree20 sorg20.dts");
addRndObj("s IceTree22 sorg22.dts");
addRndObj("s IceShrub21 sorg21.dts");
addRndObj("s IceShrub23 sorg23.dts");
addRndObj("s IceShrub24 sorg24.dts");

// Diamond Sword organics (lava)
addRndObj("s LavaPlant15 dorg15.dts 20 -3.0 0 0.8 1.5");
addRndObj("s LavaPlant16 dorg16.dts 20 -3.0 0 0.8 1.5");
addRndObj("s LavaPlant17 dorg17.dts 20 -3.0 1 0.8 1.5");
addRndObj("s LavaPlant18 dorg18.dts 20 -3.0 0 0.8 1.5");
addRndObj("s LavaPlant19 dorg19.dts 20 -3.0 0 0.8 1.5");

// Satckables
addRndObj("s Crate1 stackable1l.dts");
addRndObj("s Crate2 stackable1m.dts");
addRndObj("s Crate3 stackable1s.dts");
addRndObj("s Crate4 stackable2l.dts");
addRndObj("s Crate5 stackable2m.dts");
addRndObj("s Crate6 stackable2s.dts");
addRndObj("s Crate7 stackable3l.dts");
addRndObj("s Crate8 stackable3m.dts");
addRndObj("s Crate9 stackable3s.dts");
addRndObj("s Crate10 stackable4l.dts");
addRndObj("s Crate11 stackable4m.dts");
addRndObj("s Crate12 stackable5l.dts");
addRndObj("s Crate13 stackable5m.dts");

addRndObj("s ScoutWreckage vehicle_air_scout_wreck.dts");
addRndObj("s TankWreckage vehicle_land_assault_wreck.dts");

addRndObj("s LightMaleHumanArmor light_male.dts");
addRndObj("s MediumMaleHumanArmor medium_male.dts");
addRndObj("s HeavyMaleHumanArmor heavy_male.dts");
addRndObj("s LightFemaleHumanArmor light_female.dts");
addRndObj("s MediumFemaleHumanArmor medium_female.dts");
addRndObj("s HeavyFemaleHumanArmor heavy_male.dts");
addRndObj("s LightMaleBiodermArmor bioderm_light.dts");
addRndObj("s MediumMaleBiodermArmor bioderm_medium.dts");
addRndObj("s HeavyMaleBiodermArmor bioderm_heavy.dts");

addRndObj("s BEPlug bmiscf.dts");
addRndObj("s DSPlug dmiscf.dts");
addRndObj("s BDPlug xmiscf.dts");
addRndObj("s PHPlug pmiscf.dts");
addRndObj("s SWPlug smiscf.dts");

addRndObj("s Base statue_base.dts");
addRndObj("s HeavyMaleStatue statue_hmale.dts");
addRndObj("s LightFemaleStatue statue_lfemale.dts");
addRndObj("s LightMaleStatue statue_lmale.dts");
addRndObj("s Plaque statue_plaque.dts");

addRndObj("s LushRock6 brock6.dif");
addRndObj("s LushRock7 brock7.dif");
addRndObj("s LushRock8 brock8.dif");
addRndObj("s LushRockA brocka.dif");
addRndObj("s LushRockC brockc.dif");

addRndObj("s LushSpire1 bspir1.dif");
addRndObj("s LushSpire2 bspir2.dif");
addRndObj("s LushSpire3 bspir3.dif");
addRndObj("s LushSpire4 bspir4.dif");
addRndObj("s LushSpire5 bspir5.dif");

// example objects
// Blood Eagle Forest (lush)
// created by Tim 'Zear' Hammock for the map Fountains of Paradise

// addRndObj("s FoPS23 borg23.dts 60 0 1 2.8 4.6");
// addRndObj("s FoPS25 borg25.dts 40 0 R 2.8 4.5");
// a variety of shrubs
// addRndObj("v FoPShrubs1 2 FoPS23 1 FoPS25 2");

// addRndObj("s FoPT17 borg17.dts 50 -2 0.6 1 2");
// addRndObj("s FoPT16 borg16.dts 40 -2 0.3 1 3");
// addRndObj("s FoPT18 borg18.dts 30 -2 0.0 1 3");
// addRndObj("s FoPT19 borg19.dts 30 -2 0.0 1 3");
// addRndObj("s FoPT18a borg18.dts 30 -2 R 2 4");
// addRndObj("s FoPT19a borg19.dts 30 -2 R 2 4");
// // a variety of trees
// addRndObj("v FoPTrees1 6 FoPT17 1 FoPT16 2 FoPT18 1 FoPT19 1 FoPT18a 1 FoPT19a 1");

// // I used compound objects for rocks so I could randomize all rotation axes
// addRndObj("c FoPRock6 nul 75 0 1 0.8 1.4 1 0 brock6.dif 0 0 0 R R R 1.3 1.3 1.3");
// addRndObj("c FoPRock7 nul 75 0 1 0.8 1.4 1 0 brock7.dif 0 0 0 R R R 1.3 1.3 1.3");
// addRndObj("c FoPRock8 nul 75 0 1 0.8 1.4 1 0 brock8.dif 0 0 0 R R R 0.7 0.7 0.7");
// // a variety of rocks
// addRndObj("v FoPRocks1 3 FoPRock6 1 FoPRock7 1 FoPRock8 1");

// // a variety of varieties to make a forest (be careful of variety recursion)
// addRndObj("v FoPForest1 3 FoPShrubs1 1 FoPRocks1 1 FoPTrees1 2");

// // once the above are defined, this one line of code populates
// //   the whole mission...  :]
// rndObjMission(FoPForest1, 150);

