// By Tim 'Zear' Hammock, May 2001

// rndObjArea() is adapted from randomOrg(), but it may not be very
//    recognizable anymore... Thanks Dynamix  :)

function rndObjMission(%objName, %num, %spacing, %statSpacing, %overlap, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup)
{
   if(%overlap $= "")
      %overlap = 0.1;

   %area = MissionArea.getArea();
   %x = getWord(%area, 0) - (getWord(%area, 2) * %overlap);
   %y = getWord(%area, 1) - (getWord(%area, 3) * %overlap);
   %w = getWord(%area, 2) + (2 * getWord(%area, 2) * %overlap);
   %h = getWord(%area, 3) + (2 * getWord(%area, 3) * %overlap);

   rndObjArea(%objName, %num, %x, %y, %w, %h, %spacing, %statSpacing, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup);
}

function rndObjRadius(%objName, %num, %radius, %x, %y, %spacing, %statSpacing, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup)
{
   if(%x $= "" || %y $= "")
   {
      %ctr = LocalClientConnection.camera.getPosition();
      %x = getWord(%ctr, 0);
      %y = getWord(%ctr, 1);
   }
   
   %x = %x - %radius;
   %w = 2 * %radius;
   %y = %y - %radius;
   %h = 2 * %radius;
   
   rndObjArea(%objName, %num, %x, %y, %w, %h, %spacing, %statSpacing, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup);
}

function rndObjArea(%objName, %num, %x, %y, %w, %h, %spacing, %statSpacing, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup)
{
   // make sure a defined object of that name exists
   if ($RndObj[%objName] $= "")
   {
      error("There is no static shape named" SPC %objName);
      error("Objects not placed");
      return;
   }

   if(%spacing $= "")
      %spacing = 5.0; //min meters between center of obj and a non-static obj

   if(%statSpacing $= "")
      %statSpacing = 1.0; //min meters between center of obj and a static obj

   if(%xyFactor $= "")
      %xyFactor = 1.0; // global mult to affect the 'fatness' of the objects

   if(%zFactor $= "")
      %zFactor = 1.0; // global mult to affect the height of the objects

   if(%lowerbound $= "")
      %lowerbound = 0; // global banding

   if(%upperbound $= "")
      %upperbound = 1000; // global banding

   // allow for specifying a specific parent group, but default to default
   if(%parentGroup $= "")
      %parentGroup = "Randomstatics";

   // make sure we have that parent group
   %nominalParent = "MissionGroup";
   for(%g = 0; getword(%parentGroup, %g) !$= ""; %g++)
   {
      %thisgrp = getword(%parentGroup, %g);

      if(!isObject(%thisgrp))
      {
         %tmpgrp = new simGroup(%thisgrp);
         %nominalParent.add(%tmpgrp);
      }
      %nominalParent = %thisgrp;
   }
   %parentGroup = %nominalParent;

   //create an appropriate grp to place the objects into within the parent grp
   %group = new simGroup(rndGroupCheck("Rnd_"@%objName));
   %parentGroup.add(%group);
   %parentGroup = %group;

   // this is the object to add
   %obj = $RndObj[%objName];

   echo("Attempting to place random objects at (" @ %x @ "," @ %y @ "," @
                                                    %w @ "," @ %h @ "," @ ")");
   echo("Placing " @ %num SPC %objName @ " objects");
   echo("Spacing = " @ %spacing @ "Stat Spacing = " @ %statSpacing);

   // check for a variety
   %objType = getSubStr(getWord(%obj, 0), 0, 1);
   if(%objType $= "v" || %objType $= "V")
   {
      // it is a variety; special handling

      // determine total of child object weights
      %childCount = getWord(%obj, 2);
      %weightTotal = 0;
      for(%i = 0; %i < %childCount; %i++)
         %weightTotal += getWord(%obj, 4 + (%i * 2));

      // arrive at a weight multiplier
      //    %dist * <child weight> = number of that child to create
      %dist = %num / %weightTotal;

      // add appropriate number of each child
      for(%i = 0; %i < %childCount; %i++)
      {
         %thisChild = getWord(%obj, 3 + (%i * 2));
         %thisNum = getWord(%obj, 4 + (%i * 2)) * %dist;
         rndObjArea(%thisChild, %thisNum, %x, %y, %w, %h, %spacing, %statSpacing, %xyFactor, %zFactor, %lowerbound, %upperbound, %parentGroup);
      }
      return;
   }

   %shapeFileName = getWord(%obj, 2);

   %maxSlope = getWord(%obj, 3);
   if (%maxSlope $= "")
      %maxSlope = 40;

   %zOffset = getWord(%obj, 4);
   if (%zOffset $= "")
      %zOffset = 0;

   %slopeWithTerrain = getWord(%obj, 5);
   if (%slopeWithTerrain $= "")
      %slopeWithTerrain = false;

   %minScale = getWord(%obj, 6);
   %maxScale = getWord(%obj, 7);

   %orgCount = %num;

   //theoretically, a thorough number of retries
   while((%orgCount > 0) && (%retries < (15000 / %maxSlope)))
   {
      //find a random location - forget tiles
      %thisx = %x + (mFloor(getRandom()* %w * 1000)/1000);
      %thisy = %y + (mFloor(getRandom()* %h * 1000)/1000);

      %start = %thisx @ " " @ %thisy @ " 2000";
      %end = %thisx @ " " @ %thisy @ " -1";
      %ground = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);
      %z = getWord(%ground, 3);

      if (%z > %upperbound || %z < %lowerbound)
      {
         %retries++;
         continue;
      }

      %z += %zOffset;
      %position = %thisx @ " " @ %thisy @ " " @ %z;


      // get normal from both sides of the square
      %start = %thisx + 2 @ " " @ %thisy @ " 2000";
      %end = %thisx + 2 @ " " @ %thisy @ " -1";
      %hit1 = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);

      %start = %thisx - 2 @ " " @ %thisy @ " 2000";
      %end = %thisx - 2 @ " " @ %thisy @ " -1";
      %hit2 = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);

      %norm1 = getWord(%hit1, 4) @ " " @ getWord(%hit1, 5) @ " " @ getWord(%hit1, 6);
      %norm2 = getWord(%hit2, 4) @ " " @ getWord(%hit2, 5) @ " " @ getWord(%hit2, 6);

      //if either side of tile has greater slope than allowed, move on.
      %angNorm1 = getTerrainAngle(%norm1);
      %angNorm2 = getTerrainAngle(%norm2);
      if ((getTerrainAngle(%norm1) > %maxSlope) ||
          (getTerrainAngle(%norm2) > %maxslope))
      {
         %retries++;
         continue;
      }

      %terrainNormal = VectorAdd(%norm1, %norm2);
      %terrainNormal = VectorNormalize(%terrainNormal);

      //scale it
      %scaleMin = 8;  //default min
      %scaleMax = 14; //default max
      if(%minScale)
         %scaleMin = %minScale * 10;
      if(%maxScale)
         %scaleMax = %maxScale * 10;
      %scaleInt = getRandom(%scaleMin, %scaleMax);
      %scale = %scaleInt/10;

      // factors added below to allow 'meta' scaling
      %scaleXY = %scale * %xyFactor;
      %scaleZ = %scale * %zFactor;

      %evenScale = %scaleXY SPC %scaleXY SPC %scaleZ;
      %maxScale = (%scaleXY > %scaleZ)?%scaleXY:%scaleZ;

      //search surroundings for non-static obstacles. If obstructed, move on.
      InitContainerRadiusSearch(%position, %spacing * %maxScale,
                                           $TypeMasks::VehicleObjectType |
                                           $TypeMasks::MoveableObjectType |
                                           $TypeMasks::StaticShapeObjectType |
                                           $TypeMasks::ForceFieldObjectType |
                                           $TypeMasks::TurretObjectType |
                                           $TypeMasks::InteriorObjectType |
                                           $TypeMasks::ItemObjectType);
      %this = containerSearchNext();
      if(%this)
      {
         %retries++;
         continue;
      }

      //search surroundings for static obstacles. If obstructed, move on.
      InitContainerRadiusSearch(%position, %statSpacing * %maxScale,
                                           $TypeMasks::StaticTSObjectType);
      %this = containerSearchNext();
      if(%this)
      {
         %retries++;
         continue;
      }

      //rotate it
      if(%slopeWithTerrain)
      {
         %rotAxis = vectorCross(%terrainNormal, "0 0 1");
         %rotAxis = vectorNormalize(%rotAxis);

         if(%slopeWithTerrain $= "R")
             %rotAngle = getRandom();
         else
             %rotAngle = %slopeWithTerrain;

         %rotation = %rotAxis @ " " @
                        (getTerrainAngle(%terrainNormal) * %slopeWithTerrain);
      }
      else %rotation = "1 0 0 0";
      %randomAngle = getRandom(360);
      %zrot = MatrixCreate("0 0 0", "0 0 1 " @ %randomAngle);
      %orient = MatrixCreate(%position, %rotation);
      %finalXForm = MatrixMultiply(%orient, %zrot);

      //create it
      %modelType = getSubStr(%shapeFileName, strlen(%shapeFileName)-3, 3);
      if(%modelType $= "dts")
      {
         // shapes (organics, etc.)
         %newstatic = new TSStatic() {
            position  = %position;
            rotation  = %rotation;
            scale     = %evenScale;
            shapeName = %shapeFileName;
         };
      }
      else if(%modelType $= "dif")
      {
         // interiors (rocks, etc)
     		%newstatic = new InteriorInstance() {
            position  = %position;
            rotation  = %rotation;
            scale     = %evenScale;
   			interiorFile = %shapeFileName;
   			showTerrainInside = "0";
		   };
      }
      else if(%modelType !$= "nul")
      {
         error("Bad model type for " @ %shapeFileName @ " - aborting function");
         return;
      }

      if(%modelType !$= "nul")
      {
         %group.add(%newstatic);
         %newstatic.setTransform(%finalXForm);
      }

      if(%objType $= "c" || %objType $= "C")
      {
         // compound object
         %numCompunds = getWord(%obj, 8);
         for(%cmpdLoop = 0; %cmpdLoop < %numCompunds; %cmpdLoop++)
         {
            %file2 = getWord(%obj, 10 + (%cmpdLoop * 10));
            %xPosOff = getWord(%obj, 11 + (%cmpdLoop * 10));
            %yPosOff = getWord(%obj, 12 + (%cmpdLoop * 10));
            %zPosOff = getWord(%obj, 13 + (%cmpdLoop * 10));

            %xRotOff = getWord(%obj, 14 + (%cmpdLoop * 10));
            if(%xRotOff $= "R")
               %xRotOff = getRandom(-179, 180);
            %xRotOff = 1/%xRotOff;
            %yRotOff = getWord(%obj, 15 + (%cmpdLoop * 10));
            if(%yRotOff $= "R")
               %yRotOff = getRandom(-179, 180);
            %yRotOff = 1/%yRotOff;
            %zRotOff = getWord(%obj, 16 + (%cmpdLoop * 10));
            if(%zRotOff $= "R")
               %zRotOff = getRandom(-179, 180);
            %zRotOff = 1/%zRotOff;

            %xScaleOff = getWord(%obj, 17 + (%cmpdLoop * 10));
            %yScaleOff = getWord(%obj, 18 + (%cmpdLoop * 10));
            %zScaleOff = getWord(%obj, 19 + (%cmpdLoop * 10));
            
            if(%xScaleOff <= 0 || %yScaleOff <= 0 || %zScaleOff <= 0)
            {
               error("Scale zero error");
               echo("Child "@%file2@" of compound object "@
                      %objName@"has scale offset of zero!");
               echo("Compound object will be incomplete!");
               continue;
            }

            %zrot2 = MatrixCreate(%xPosOff SPC %yPosOff SPC %zPosOff,
                                  %xRotOff SPC %yRotOff SPC %zRotOff SPC "180");
            %finalXForm2 = MatrixMultiply(%orient, %zrot2);
            %evenScale2 = (%scale * %xyFactor * %xScaleOff) SPC
                          (%scale * %xyFactor * %yScaleOff) SPC
                          (%scale * %zFactor * %zScaleOff);

            %modelType2 = getSubStr(%file2, strlen(%file2)-3, 3);
            if(%modelType2 $= "dts")
            {
               // shapes (organics, etc.)
               %newstatic2 = new TSStatic() {
                  position  = %position;
                  rotation  = %rotation;
                  scale     = %evenScale2;
                  shapeName = %file2;
               };
            }
            else if(%modelType2 $= "dif")
            {
               // interiors (rocks, etc)
           		%newstatic2 = new InteriorInstance() {
                  position  = %position;
                  rotation  = %rotation;
                  scale     = %evenScale2;
         			interiorFile =%file2;
         			showTerrainInside = "0";
      		   };

            }
            else
            {
               error("Bad model type for " @ %file2 @ " - Aborting function");
               return;
            }

            %group.add(%newstatic2);
            %newstatic2.setTransform(%finalXForm2);
         }
      }

      %orgCount--;   //dec number of shapes left to place
      %retries = 0; //reset retry counter
   }
   if (%orgCount > 0)
   {
      error("Unable to place all shapes, area saturated.");
      error("Looking for clear area " @ (%spacing * 2) @
                      " meters in diameter, with a max slope of " @ %maxSlope);
   }
   echo("Placed " @ %num - %orgCount @ " of " @ %num);
}

function rndGroupCheck(%candidate)
{
   if(!isObject(%candidate))
      return %candidate;

   %inc = 2;
   while(isObject(%candidate@"_"@%inc))
   {
      %inc++;
   }
   
   return %candidate@"_"@%inc;
}

$RndObjCount = 0;
function addRndObj(%str)
{
   %name = getWord(%str, 1);
   if($RndObj[%name] $= "")
   {
      $RndObj[%name] = %str;
      $RndObjCount++;
   }
   else if($RndObj[%name] !$= %str)
   {
      error("Random object "@%name@" already defined. Original replaced.");
      echo("Original: \""@$RndObj[%name]@"\"");
      echo("Current: \""@%str@"\"");
      $RndObj[%name] = %str;
   }
}


